package info.textgrid.lab.collatex.model.equivalences;

import java.net.URI;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.bind.annotation.XmlTransient;

/**
 * Abstract base class for the position of a token in a witness.
 * {@link #getUri()} contains the witnesses URI, subclasses provide specific
 * means to reference the individual token.
 */
@XmlRootElement
@XmlSeeAlso(TokenIndex.class)
public abstract class Position {

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((uri == null) ? 0 : uri.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Position other = (Position) obj;
		if (uri == null) {
			if (other.uri != null)
				return false;
		} else if (!uri.equals(other.uri))
			return false;
		return true;
	}

	@XmlTransient
	private URI uri;

	@XmlAttribute(required = true)
	public URI getUri() {
		return uri;
	}

	protected void setUri(final URI uri) {
		this.uri = uri;
	}

}
