package info.textgrid.lab.collatex.ui.parts;

import info.textgrid.lab.collatex.model.CollationResults;
import info.textgrid.lab.collatex.ui.parts.CollationSetEditor.ICollationEventHandler;

import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.zest.core.viewers.AbstractZoomableViewer;
import org.eclipse.zest.core.viewers.GraphViewer;
import org.eclipse.zest.core.viewers.IGraphContentProvider;
import org.eclipse.zest.core.viewers.IZoomableWorkbenchPart;
import org.eclipse.zest.core.widgets.ZestStyles;
import org.eclipse.zest.layouts.LayoutStyles;
import org.eclipse.zest.layouts.algorithms.HorizontalTreeLayoutAlgorithm;
import org.neo4j.graphdb.Transaction;

import com.google.common.base.Joiner;
import com.google.common.base.Preconditions;
import com.google.common.collect.Iterables;

import eu.interedition.collatex.graph.VariantGraph;
import eu.interedition.collatex.graph.VariantGraphEdge;
import eu.interedition.collatex.graph.VariantGraphVertex;

/**
 * Result page showing a graph representation of the results.
 */
public class VariantGraphPage extends AbstractResultPage implements IZoomableWorkbenchPart,
ICollationEventHandler {

	public static final String ID = "info.textgrid.lab.collatex.ui.parts.VariantGraphView"; //$NON-NLS-1$
	private GraphViewer graphViewer;
	private Composite control;
	private CollationSetEditor collationSetEditor;

	@Override
	public void dispose() {
		if (collationSetEditor != null)
			collationSetEditor.removeCollationListener(this);
		super.dispose();
	}

	@Override
	public Composite getControl() {
		return control;
	}

	public VariantGraphPage() {
	}

	/**
	 * Create contents of the view part.
	 * @param parent
	 */
	@Override
	public void createControl(final Composite parent) {
		final Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(1, false));
		setViewer(new GraphViewer(container, SWT.NONE));
		final Control control = getViewer().getControl();
		control.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		getViewer().setContentProvider(new IGraphContentProvider() {

			// private VariantGraph variantGraph;

			@Override
			public void inputChanged(final Viewer viewer,
					final Object oldInput, final Object newInput) {
				// Preconditions.checkArgument(newInput instanceof VariantGraph,
				// "Viewer requires a variant graph, not a {0}", newInput
				// .getClass().getSimpleName());
				// this.variantGraph = (VariantGraph) newInput;
			}

			@Override
			public void dispose() {
				// TODO Auto-generated method stub

			}

			@Override
			public Object[] getElements(final Object input) {
				Preconditions.checkArgument(input instanceof CollationResults);
				final VariantGraph graph = ((CollationResults) input)
						.getVariantGraph();
				final Transaction transaction = graph.getDatabase().beginTx();
				try {
					final VariantGraphEdge[] edges = Iterables.toArray(
							graph
							.join().rank().edges(), VariantGraphEdge.class);
					transaction.success();
					return edges;
				} finally {
					transaction.finish();
				}
			}

			@Override
			public Object getSource(final Object rel) {
				Preconditions.checkArgument(rel instanceof VariantGraphEdge);
				return ((VariantGraphEdge) rel).from();
			}

			@Override
			public Object getDestination(final Object rel) {
				Preconditions.checkArgument(rel instanceof VariantGraphEdge);
				return ((VariantGraphEdge) rel).to();
			}
		});

		getViewer().setLabelProvider(new LabelProvider() {

			@Override
			public String getText(final Object element) {
				if (element instanceof VariantGraphVertex) {
					final VariantGraphVertex vertex = (VariantGraphVertex) element;
					return VariantGraphVertex.TO_CONTENTS.apply(vertex);
					// final Set<Token> tokens = vertex.tokens();
					// if (tokens.isEmpty())
					//						return ""; //$NON-NLS-1$
					// else
					// return Joiner.on(' ').join(tokens.iterator());

				} else if (element instanceof VariantGraphEdge) {
					final VariantGraphEdge edge = (VariantGraphEdge) element;
					if (!edge.from().witnesses().equals(edge.witnesses()))
						return Joiner.on(", ").join(edge.witnesses()); //$NON-NLS-1$
				}
				return null;
			}
		});
		getViewer().setLayoutAlgorithm(new HorizontalTreeLayoutAlgorithm(
				LayoutStyles.NO_LAYOUT_NODE_RESIZING));
		getViewer().setNodeStyle(ZestStyles.NONE);
		getViewer().setConnectionStyle(ZestStyles.CONNECTIONS_DIRECTED);

		this.control = container;

		if (getCollationSetEditor() != null
				&& getCollationSetEditor().getCollationResults() != null)
			newCollationResults(getCollationSetEditor().getCollationResults());
	}


	@Override
	public AbstractZoomableViewer getZoomableViewer() {
		return getViewer();
	}

	public void setInput(final VariantGraph variantGraph) {
		if (getViewer() != null)
			getViewer().setInput(variantGraph);
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub

	}

	@Override
	protected GraphViewer getViewer() {
		return graphViewer;
	}

	protected void setViewer(final GraphViewer graphViewer) {
		this.graphViewer = graphViewer;
	}

}
